resultado_rolagem_teste = ('Falha', 'Sucesso')
resultado_rolagem_ataque = ('Erro', 'Acerto', 'Crítico')

tipos_moedas = ['cp', 'sp', 'ep', 'gp', 'pp']

valor_moedas = {
    'cp': 1/100,
    'sp': 1/10,
    'ep': 1/2,
    'gp': 1,
    'pp': 10
}